'use strict';

class AbstractContainer {
  constructor(params) {
    this._config = this._buildConfig(params);
    this._store  = {};
  }

  register(key, value, options) {
    let args = this._buildRegisterOptions(options);

    this._log(`Registering new component`);
    this._log(JSON.stringify({ key: key, value: value, options: args }, null, '\t'));

    if ((this._store[key] !== undefined) && (args.allowOverwrite !== true)) {
      throw new Error(`Duplicate declaration of ${key} found`);
    } else {
      this._store[key] = {
        component: value,
        instantiate: {
          enabled: args.instantiate.enabled,
          options: args.instantiate.options
        }
      };
      return value;
    }
  }

  fetch(key, fn) {
    const value      = this._store[key];
    const ifNotFound = (fn ? fn : (label) => { throw new Error(`Could not find ${label}`)});

    this._log(`fetching ${key}`);

    if (value) {
      this._log(`found component for ${key}`);
      if (value.instantiate.enabled) {
        return new value.component(value.instantiate.options);
      } else {
        return value.component;
      }
    } else {
      this._log(`could not find component for ${key}`);
      return ifNotFound(key);
    }
  }

  _buildConfig(params) {
    let opts      = (params || {});
    const object  = {};
    object.debug  = opts.debug || false;
    object.logger = opts.logger || console;
    return object;
  }

  _buildRegisterOptions(params) {
    const opts       = (params || {});
    opts.instantiate = (opts.instantiate || {});
    return opts;
  }

  _log(text) {
    if (this._config.debug) {
      return this._config.logger.log("[abstract-container] " + text);
    }
  }
}

module.exports = AbstractContainer;