"use strict";
const assert            = require("assert");
const AbstractContainer = require('../lib/abstract-container');

class MyContainer extends AbstractContainer {}

const subject = new MyContainer({ debug: false });

describe("AbstractContainer", function() {
  describe("#_store", function() {
    it("returns an object where objects are stored", function() {
      assert.equal(typeof subject._store, 'object');
    });
  });

  describe("#register", function() {
    it("without a previously set value for that key - it registers an object", function() {
      const component = { foo: 'bar' };
      const key       = 'component:demo';

      subject.register(key, component);
      assert.deepEqual(subject._store[key]['component'], component);
    });

    it("with a previously set value for that key - it throws an error", function() {
      const key = 'component:demo';
      const fn  = (container, label) => {
        return () => {
          container.register(label, value)
        };
      };

      assert.throws(fn(subject, key), Error, `Duplicate declaration for ${key} found`);
    });

    it("with a previously set value and the setting to overwrite it - it overwrites the registered object", function() {
      const key          = 'component:demo';
      const newComponent = { 'foo': 'second-bar' };

      subject.register(key, newComponent, { allowOverwrite: true });

      assert.deepEqual(subject._store[key]['component'], newComponent);
    });
  });

  describe("#fetch", function() {
    it("with a registered record - returns the record", function() {
      assert.ok(subject.fetch("component:demo"));
    });

    it("without a registered record and without a block - throws an error", function() {
      const key = 'key';
      const fn = (container) => {
        return () => {
          container.fetch(key);
        };
      };

      assert.throws(fn(subject), Error, `Could not find ${key}`);
    });

    it("without a registered record and with a block - calls the block with the key as an argument", function() {
      const block = (label) => label;

      assert.equal(subject.fetch('foobar', block), 'foobar');
    });
  });
});